-- aChat, BasicChatMods, Alza code + personal code

DEFAULT_CHATFRAME_ALPHA = 0
--Opacity of chat frames when the mouse is over them.
--Default is 0.25.

CHAT_FRAME_FADE_OUT_TIME = 0
--Seconds to wait before fading out chat frames the mouse moves out of.
--Default is 2.

CHAT_TAB_HIDE_DELAY = 0
--Seconds to wait before fading out chat tabs the mouse moves out of.
--Default is 1.

CHAT_FRAME_TAB_SELECTED_MOUSEOVER_ALPHA = 1
CHAT_FRAME_TAB_SELECTED_NOMOUSE_ALPHA = 0
--Opacity of the currently selected chat tab.
--Defaults are 1 and 0.4.

CHAT_FRAME_TAB_ALERTING_MOUSEOVER_ALPHA = 1
CHAT_FRAME_TAB_ALERTING_NOMOUSE_ALPHA = 0
--Opacity of currently alerting chat tabs.
--Defaults are 1 and 1.

CHAT_FRAME_TAB_NORMAL_MOUSEOVER_ALPHA = 1
CHAT_FRAME_TAB_NORMAL_NOMOUSE_ALPHA = 0
--Opacity of non-selected, non-alerting chat tabs.
--Defaults are 0.6 and 0.2.

local hooks = {}
local chatEvents = {
	"CHAT_MSG_CHANNEL_JOIN", 
	"CHAT_MSG_CHANNEL_LEAVE", 
	"CHAT_MSG_CHANNEL_NOTICE", 
	"CHAT_MSG_CHANNEL_NOTICE_USER", 
	"CHAT_MSG_CHANNEL_LIST"
}

local function HideForever(f)
	f:SetScript("OnShow", f.Hide)
	f:Hide()
end

local function AddMessage(frame, text, r, g, b, id)
	local editMessage = true
	for i, v in ipairs(chatEvents) do
		if(event == v) then
			editMessage = false
			break
		end
	end
	
	if(editMessage) then
		text = text:gsub("%[Guild%]", "g")
		text = text:gsub("%[Party%]", "p")
		text = text:gsub("%[Party Leader%]", "P")
		text = text:gsub("%[Dungeon Guide%]", "P")
		text = text:gsub("%[Raid%]", "r")
		text = text:gsub("%[Raid Leader%]", "RL")
		text = text:gsub("%[Raid Warning%]", "RW")
		text = text:gsub("%[Officer%]", "o")
		text = text:gsub("%[Battleground%]", "bg")
		text = text:gsub("%[Battleground Leader%]", "BG")
		text = text:gsub("%[(%d+)%..-%]", "%1")
		text = text:gsub("(|Hplayer.*|h) whispers", "From %1")
		text = text:gsub("To (|Hplayer.*|h)", "To %1")
		text = text:gsub("(|Hplayer.*|h) says", "%1")
		text = text:gsub("(|Hplayer.*|h) yells", "%1")
		text = text:gsub("(|Hplayer.*|h)", "%1")
		text = text:gsub("|H(.-)|h%[(.-)%]|h", "|H%1|h%2|h") -- Remove brackets around names
	end
	
	return hooks[frame](frame, text, r, g, b, id)
end

local scrollFunc = function(frame) frame:GetParent():ScrollToBottom() frame:Hide() end
local showFunc = function(frame)
	local n = frame:GetName()
	if frame:GetCurrentScroll() > 0 then
		_G[n.."ButtonFrameBottomButton"]:Show()
	else
		_G[n.."ButtonFrameBottomButton"]:Hide()
	end
end

for i = 1, NUM_CHAT_WINDOWS do
	local frame = _G[("ChatFrame%d"):format(i)]
	
	--Hide buttons
	HideForever(frame.buttonFrame)
	
	--Disable chat fading
	frame:SetFading(false)

	--Font
	frame:SetFont("Fonts\\ARIALN.ttf", 13, "THINOUTLINE")
	frame:SetShadowColor(0, 0, 0, 0)

	--Resize!
	frame:SetMinResize(0,0)
	frame:SetMaxResize(0,0)

	--Move editbox to top
	frame.editBox:ClearAllPoints()
	frame.editBox:SetPoint("BOTTOMLEFT",  _G.ChatFrame1, "TOPLEFT", -5, 18)
	frame.editBox:SetPoint("BOTTOMRIGHT", _G.ChatFrame1, "TOPRIGHT", 5, 18)
	frame.editBox:SetFont("Fonts\\ARIALN.ttf", 13, "THINOUTLINE")
	frame.editBox.header:SetFont("Fonts\\ARIALN.ttf", 13, "THINOUTLINE")
	frame.editBox.header:SetShadowColor(0, 0, 0, 0)
	frame.editBox:SetShadowColor(0, 0, 0, 0)
	
	--Enable moving in editbox without holding alt
	frame.editBox:SetAltArrowKeyMode(nil)

	--Custom editbox background!
	local x=({_G["ChatFrame"..i.."EditBox"]:GetRegions()})
	x[9]:SetAlpha(0)
	x[10]:SetAlpha(0)
	x[11]:SetAlpha(0)

	local ebg = CreateFrame("Frame", nil, frame.editBox)
	ebg:SetBackdrop({ 
		bgFile = FreeUI.backdrop, 
		edgeFile = FreeUI.backdrop,
		edgeSize = 1,
	})
	ebg:SetBackdropColor(0, 0, 0, .4)
	ebg:SetPoint("TOPLEFT", frame.editBox, 4, -4)
	ebg:SetPoint("BOTTOMRIGHT", frame.editBox, -4, 4)
	ebg:SetFrameStrata("BACKGROUND")
	ebg:SetFrameLevel(0)

	--From Tukui
	local function colorize(r, g, b)
		ebg:SetBackdropBorderColor(r, g, b)
	end

	hooksecurefunc("ChatEdit_UpdateHeader", function()
		local type = frame.editBox:GetAttribute("chatType")
		if ( type == "CHANNEL" ) then
			local id = GetChannelName(frame.editBox:GetAttribute("channelTarget"))
			if id == 0 or id == 1 or id == 2 or id == 3 then
				colorize(0, 0, 0)
			else
				colorize(ChatTypeInfo[type..id].r,ChatTypeInfo[type..id].g,ChatTypeInfo[type..id].b)
			end
		elseif (type == "SAY") then
			colorize(0, 0, 0)
		else
			colorize(ChatTypeInfo[type].r,ChatTypeInfo[type].g,ChatTypeInfo[type].b)
		end
	end)

	--Hide background
	for j = 1, #CHAT_FRAME_TEXTURES do
		_G["ChatFrame"..i..CHAT_FRAME_TEXTURES[j]]:SetTexture(nil)
	end
	
	--Hide the new editbox "ghost"
	_G[("ChatFrame%sEditBoxLeft"):format(i)]:SetAlpha(0)
	_G[("ChatFrame%sEditBoxRight"):format(i)]:SetAlpha(0)
	_G[("ChatFrame%sEditBoxMid"):format(i)]:SetAlpha(0)
	
	--Allow the chatframe to be moved flush with the left and bottom sides of the screen
	frame:SetClampRectInsets(0, 0, 0, 0)

	--Shorten channel names
	hooks[frame] = frame.AddMessage
	frame.AddMessage = AddMessage
end

--Hide buttons
HideForever(ChatFrameMenuButton)
HideForever(FriendsMicroButton)
HideForever(GeneralDockManagerOverflowButton)

--Make channels sticky
ChatTypeInfo.SAY.sticky = 1
ChatTypeInfo.EMOTE.sticky = 1
ChatTypeInfo.YELL.sticky = 1
ChatTypeInfo.PARTY.sticky = 1
ChatTypeInfo.GUILD.sticky = 1
ChatTypeInfo.OFFICER.sticky = 1
ChatTypeInfo.RAID.sticky = 1
ChatTypeInfo.RAID_WARNING.sticky = 1
ChatTypeInfo.BATTLEGROUND.sticky = 1
ChatTypeInfo.WHISPER.sticky = 1
ChatTypeInfo.BN_WHISPER.sticky = 1
ChatTypeInfo.CHANNEL.sticky = 1

--Don't want bread in my editbox
BNToastFrame:HookScript("OnShow", function()
	BNToastFrame:ClearAllPoints()
	BNToastFrame:SetPoint("TOPLEFT", ChatFrame1, 0, 100)
end)

--Make toaster prettier
FreeUI.CreateBD(BNToastFrame)
BNToastFrameCloseButton:SetAlpha(0)

--Background
local cbg = CreateFrame("Frame", nil, ChatFrame1)
FreeUI.CreateBD(cbg)
cbg:SetPoint("TOPLEFT", ChatFrame1, -1, 1)
cbg:SetPoint("BOTTOMRIGHT", ChatFrame1, 1, -1)
cbg:SetFrameStrata("BACKGROUND")
cbg:SetFrameLevel(0)
cbg:Hide()

--Handle the background.
local f = CreateFrame("Frame")
f:SetBackdrop({
	bgFile = FreeUI.backdrop,
	edgeFile = FreeUI.glow,
	edgeSize = 3,
	insets = {left = 3, right = 3, top = 3, bottom = 3},
})
f:SetBackdropColor(.4, .6, 1)
f:SetBackdropBorderColor(.4, .6, 1)
f:SetAlpha(0)
f:SetWidth(8)
f:SetHeight(8)
f:SetPoint("BOTTOMLEFT")
f:EnableMouse(true)

f:SetScript("OnMouseDown", function(self, button)
	if button == "LeftButton" then
		if cbg:IsShown() then
			cbg:Hide()
		else
			cbg:Show()
		end
	elseif button == "RightButton" then
		ToggleFrame(ChatMenu)
	end
end)

f:SetScript("OnEnter", function()
	f:SetAlpha(1)
	FreeUI.CreatePulse(f)
end)

f:SetScript("OnLeave", function()
	f:SetScript("OnUpdate", nil)
	f:SetAlpha(0)
end)

--Tell target
SlashCmdList["TELLTARGET"] = function(s)
	if(UnitExists("target") and UnitName("target") and UnitIsPlayer("target") and GetDefaultLanguage("player")==GetDefaultLanguage("target"))then
		SendChatMessage(s, "WHISPER", nil, UnitName("target"))
	end
end
SLASH_TELLTARGET1 = "/tt"
SLASH_TELLTARGET2 = "/ее"

--New class colours without causing taint
function GetColoredName(event, _, senderName, _, _, _, _, _, channelNumber, _, _, _, senderGUID)
	local chatType = event:sub(10)
	if chatType:sub(1, 7) == "WHISPER" then
		chatType = "WHISPER"
	elseif chatType:sub(1, 7) == "CHANNEL" then
		chatType = "CHANNEL" .. channelNumber
	end
	local info = ChatTypeInfo[chatType]
	if info and senderGUID and info.colorNameByClass and senderGUID ~= "" then
		local _, class = GetPlayerInfoByGUID(senderGUID)
		if class then
			local color = CUSTOM_CLASS_COLORS[class]
			if color then
				return ("|cff%02x%02x%02x%s|r"):format(color.r * 255, color.g * 255, color.b * 255, senderName)
			end
		end
	end
	return senderName
end

--Copy when you shift click tab
local lines = {}

local frame = CreateFrame("Frame", "BCMCopyFrame", UIParent)
frame:SetWidth(ChatFrame1:GetWidth() + 35)
frame:SetHeight(350)
frame:SetPoint("LEFT", UIParent, "LEFT", 50, 0)
frame:SetFrameStrata("DIALOG")
frame:Hide()

FreeUI.CreateBD(frame)

local editBox = CreateFrame("EditBox", "BCMCopyBox", frame)
editBox:SetMultiLine(true)
editBox:SetMaxLetters(20000)
editBox:EnableMouse(true)
editBox:SetAutoFocus(true)
editBox:SetFont("Fonts\\ARIALN.ttf", 13, "THINOUTLINE")
editBox:SetWidth(ChatFrame1:GetWidth())
editBox:SetScript("OnEscapePressed", function() frame:Hide() wipe(lines) end)

local scrollArea = CreateFrame("ScrollFrame", "BCMCopyScroll", frame, "UIPanelScrollFrameTemplate")
scrollArea:SetPoint("TOPLEFT", frame, "TOPLEFT", 8, -8)
scrollArea:SetPoint("BOTTOMRIGHT", frame, "BOTTOMRIGHT", -30, 8)
scrollArea:SetScrollChild(editBox)

local function GetLines(...)
	local ct = 1
	for i = select("#", ...), 1, -1 do
		local region = select(i, ...)
		if(region:GetObjectType()=="FontString") then
			lines[ct] = tostring(region:GetText())
			ct = ct + 1
		end
	end
	return ct - 1
end

local function Copy(cf)
	local _, size = cf:GetFont()
	FCF_SetChatWindowFontSize(cf, cf, 0.01)
	local lineCt = GetLines(cf:GetRegions())
	local text = table.concat(lines, "\n", 1, lineCt)
	FCF_SetChatWindowFontSize(cf, cf, size)
	frame:Show()
	editBox:SetText(text)
end

ChatFrame1Tab:HookScript("OnClick", function()
	if(IsShiftKeyDown()) then
		Copy(ChatFrame1)
	end
end)