-- Base code by Alza, most code personal.

local Scale = 0.9

Minimap:ClearAllPoints()
Minimap:SetPoint("BOTTOMRIGHT", UIParent, "BOTTOMRIGHT", -50 / Scale, 50 / Scale)
Minimap:SetMaskTexture"Interface\\Buttons\\WHITE8X8"
MinimapCluster:SetScale(Scale)

minibg = CreateFrame("Frame", nil, Minimap)
minibg:SetBackdrop({
	edgeFile = FreeUI.backdrop, 
	edgeSize = 1, 
})
minibg:SetBackdropBorderColor(0, 0, 0)
minibg:SetPoint("TOPLEFT", Minimap, -1, 1)
minibg:SetPoint("BOTTOMRIGHT", Minimap, 1, -1)
minibg:SetFrameStrata("BACKGROUND")

minibg:SetScript("OnEvent", function(self, event, ...)
	local inv = CalendarGetNumPendingInvites()
	local mail = HasNewMail()
	if inv > 0 and mail then -- New invites and mail
		minibg:SetBackdropBorderColor(1, .5, 0)
	elseif inv > 0 and not mail then -- New invites and no mail
		minibg:SetBackdropBorderColor(1, 30/255, 60/255)
	elseif inv==0 and mail then -- No invites and new mail
		minibg:SetBackdropBorderColor(0, 1, 0)
	else -- None of the above
		minibg:SetBackdropBorderColor(0, 0, 0)
	end
end)

minibg:RegisterEvent("CALENDAR_UPDATE_PENDING_INVITES")
minibg:RegisterEvent("UPDATE_PENDING_MAIL")
minibg:RegisterEvent("PLAYER_ENTERING_WORLD")

ZoneTextString:ClearAllPoints()
ZoneTextString:SetPoint("CENTER", Minimap)
ZoneTextString:SetWidth(Minimap:GetWidth())
ZoneTextString:SetFont(FreeUI.font, 8, "OUTLINEMONOCHROME")
SubZoneTextString:SetFont(FreeUI.font, 8, "OUTLINEMONOCHROME")
PVPInfoTextString:SetFont(FreeUI.font, 8, "OUTLINEMONOCHROME")
PVPArenaTextString:SetFont(FreeUI.font, 8, "OUTLINEMONOCHROME")

local bu = MinimapZoneTextButton
local text = MinimapZoneText
bu:ClearAllPoints()
bu:SetPoint("CENTER", Minimap)
bu:SetFrameStrata("HIGH")
bu:EnableMouse(false)
bu:SetAlpha(0)
text:SetFont(FreeUI.font, 8 / Scale, "OUTLINEMONOCHROME")
text:SetShadowColor(0, 0, 0, 0)

Minimap:HookScript("OnEnter", function()
	bu:SetAlpha(1)
end)
Minimap:HookScript("OnLeave", function()
	bu:SetAlpha(0)
end)

local function CaptureBar()
	for i = 1, NUM_EXTENDED_UI_FRAMES do
		local bar = _G["WorldStateCaptureBar"..i]
		if bar and bar:IsShown() then
			bar:ClearAllPoints()
			bar:SetPoint("TOP", UIParent, "TOP", 0, -120)
		end
	end
end

hooksecurefunc("WorldStateAlwaysUpFrame_Update", CaptureBar)

do
	local frames = {
		"MiniMapInstanceDifficulty",
		"MiniMapBattlefieldBorder",
		"MiniMapVoiceChatFrame",
		"MiniMapWorldMapButton",
		"MiniMapMailBorder",
		"MinimapBorderTop",
		"MinimapNorthTag",
		"MiniMapTracking",
		"MinimapZoomOut",
		"MinimapZoomIn",
		"MinimapBorder",
	}

	for i = 1, #frames do
		_G[frames[i]]:Hide()
		_G[frames[i]].Show = FreeUI.dummy
	end
end

Minimap:EnableMouseWheel(true)
Minimap:SetScript("OnMouseWheel", function(_, zoom)
	if(zoom>0) then
		Minimap_ZoomIn()
	else
		Minimap_ZoomOut()
	end
end)

Minimap:SetScript("OnMouseUp", function(self, button)
	if(button=="RightButton") then
		ToggleDropDownMenu(1, nil, MiniMapTrackingDropDown, "cursor", 0, 0)
	elseif(button=="MiddleButton") then
		if(not CalendarFrame) then LoadAddOn("Blizzard_Calendar") end
		Calendar_Toggle()
	else
		Minimap_OnClick(self)
	end
end)

MinimapCluster:EnableMouse(false)

GameTimeFrame:ClearAllPoints()
GameTimeFrame:SetPoint("TOPRIGHT", Minimap, "TOPRIGHT", -2, -2)

local bu = _G["GameTimeFrame"]
bu:SetWidth(24)
bu:SetHeight(24)
bu:SetHitRectInsets(0, 0, 0, 0)

select(5, GameTimeFrame:GetRegions()):SetTextColor(1, 1, 1)
select(5, GameTimeFrame:GetRegions()):SetFont(FreeUI.font, 8 / Scale, "OUTLINEMONOCHROME")
select(5, GameTimeFrame:GetRegions()):ClearAllPoints()
select(5, GameTimeFrame:GetRegions()):SetPoint("CENTER", bu, "CENTER", 0, 1)

bu:SetNormalTexture("")
bu:SetPushedTexture("")
bu:SetHighlightTexture("")

MiniMapBattlefieldFrame:ClearAllPoints()
MiniMapBattlefieldFrame:SetPoint("TOP", Minimap, "TOP", 0, 3)

MiniMapMailIcon:SetTexture(nil)

MiniMapLFGFrameBorder:SetAlpha(0)
MiniMapLFGFrame:ClearAllPoints()
MiniMapLFGFrame:SetPoint("BOTTOMRIGHT", Minimap)

TicketStatusFrame:ClearAllPoints()
TicketStatusFrame:SetPoint("TOPLEFT", UIParent, "TOPLEFT", 50, -50)
FreeUI.CreateBD(TicketStatusFrameButton)

LFDSearchStatus:ClearAllPoints()
LFDSearchStatus:SetPoint("BOTTOMRIGHT", Minimap, "BOTTOMLEFT", -4.5, -1.5)

StreamingIcon:ClearAllPoints()
StreamingIcon:SetPoint("BOTTOMLEFT", Minimap, "BOTTOMLEFT")

TimeManagerClockButton:Hide()

local rd = CreateFrame("Frame", nil, Minimap)
rd:SetPoint("TOPLEFT", Minimap, "TOPLEFT", 0, -2)
rd:RegisterEvent("PLAYER_ENTERING_WORLD")
rd:RegisterEvent("PLAYER_DIFFICULTY_CHANGED")
rd:SetSize(30, 22)

local rdt = FreeUI.CreateFS(rd, 8 / Scale)
rdt:SetPoint("CENTER")

local function diff()
	local inInstance, instanceType = IsInInstance()
	local _, _, difficultyIndex, _, _, dynamicDifficulty, isDynamic = GetInstanceInfo()

	if inInstance and instanceType == "raid" then
		if (isDynamic and difficultyIndex == 1 and dynamicDifficulty == 0) or (not isDynamic and difficultyIndex == 1) then
			rdt:SetText("10")
		elseif (isDynamic and (difficultyIndex == 3 and dynamicDifficulty == 0) or (difficultyIndex == 1 and dynamicDifficulty == 1)) or (not isDynamic and difficultyIndex == 3) then
			rdt:SetText("10H")
		elseif (isDynamic and difficultyIndex == 2 and dynamicDifficulty == 0) or (not isDynamic and difficultyIndex == 2) then
			rdt:SetText("25")
		elseif (isDynamic and (difficultyIndex == 2 and dynamicDifficulty == 1) or (difficultyIndex == 4)) or (not isDynamic and difficultyIndex == 4) then
			rdt:SetText("25H")
		end
	elseif inInstance and instanceType == "party" then
		if difficultyIndex == 1 then
			rdt:SetText("5")
		elseif difficultyIndex == 2 then
			rdt:SetText("5H")
		end
	else
		rdt:SetText("")
	end
end

rd:SetScript("OnEvent", diff)