-- aTooltip by Alza, partially modified.

PVP_ENABLED = ""

local offset = UIParent:GetScale()

GameTooltip_SetDefaultAnchor = function(self, parent)
	if FreeUI.config.tooltip_cursor == true then
		self:SetOwner(parent, "ANCHOR_CURSOR")
	else
		self:SetOwner(parent, "ANCHOR_NONE")
		self:SetPoint("BOTTOMRIGHT", UIParent, "BOTTOMRIGHT", -50, 282)
	end
	self.default = 1
end

--[[ BG ]]
do
	local tooltips = {
		"GameTooltip",
		"ItemRefTooltip",
		"ShoppingTooltip1",
		"ShoppingTooltip2",
		"ShoppingTooltip3",
		"DropDownList1MenuBackdrop",
		"DropDownList2MenuBackdrop",
		"WorldMapTooltip",
		"ChatMenu",
		"EmoteMenu",
		"LanguageMenu",
		"VoiceMacroMenu",
	}

	for i = 1, #tooltips do
		local t = _G[tooltips[i]]
		t:SetBackdrop(nil)
		local bg = CreateFrame ("Frame", nil, t)
		bg:SetBackdrop({ 
			bgFile = FreeUI.backdrop, 
			edgeFile = FreeUI.backdrop,
			edgeSize = 1,
		})
		bg:SetBackdropColor(0, 0, 0, .6)
		bg:SetBackdropBorderColor(0, 0, 0)
		bg:SetWidth(t:GetWidth())
		bg:SetHeight(t:GetHeight())
		bg:SetPoint("TOPLEFT", t, -1, 1)
		bg:SetPoint("BOTTOMRIGHT", t, 1, -1)
		bg:SetFrameStrata("DIALOG")
	end
end

--[[ Statusbar ]]
function HealthBar_OnValueChanged(self)
	self:SetStatusBarColor(.3, 1, .3)
	self:SetStatusBarTexture(FreeUI.texture)
end

local sb = _G["GameTooltipStatusBar"]
sb:SetHeight(3)
sb:ClearAllPoints()
sb:SetPoint("BOTTOMLEFT", GameTooltip, "BOTTOMLEFT", 1, 1)
sb:SetPoint("BOTTOMRIGHT", GameTooltip, "BOTTOMRIGHT", -1, 1)

--[[ Unit tooltip styling ]]
local classification = {
	worldboss = "",
	rareelite = "R+",
	elite = "+",
	rare = "R",
}

local function OnTooltipSetUnit(self)
	local lines = self:NumLines()
	local name, unit = self:GetUnit()
	if(not unit) then return end

	local race = UnitRace(unit) or ""
	local level = UnitLevel(unit) or ""
	local c = UnitClassification(unit)
	local crtype = UnitCreatureType(unit)
	local unitName, unitRealm = UnitName(unit)

	if(level and level==-1) then
		if(c=="worldboss") then
			level = "|cffff0000Boss|r"
		else
			level = "|cffff0000??|r"
		end
	end

	if unitRealm then
		_G["GameTooltipTextLeft1"]:SetFormattedText("%s - %s", name, unitRealm)
	else
		_G["GameTooltipTextLeft1"]:SetText(name)
	end

	if(UnitIsPlayer(unit)) then
		local InGuild = GetGuildInfo(unit)
		if(InGuild) then
			_G["GameTooltipTextLeft2"]:SetFormattedText("%s", InGuild)
		end

		local n = InGuild and 3 or 2
		_G["GameTooltipTextLeft"..n]:SetFormattedText("%s %s", level, race)
	else
		for i = 2, lines do
			local line = _G["GameTooltipTextLeft"..i]
			local text = line:GetText() or ""
			if((level and text:find("^"..LEVEL)) or (crtype and text:find("^"..crtype))) then
				line:SetFormattedText("%s%s %s", level, classification[c] or "", crtype or "")
				break
			end
		end
	end

	--[[ Target line ]]
	local tunit = unit.."target"
	if(UnitExists(tunit) and unit~="player") then
		local r, g, b = GameTooltip_UnitColor(tunit)
		local text = ""

		if(UnitName(tunit)==UnitName("player")) then
			text = "T: > YOU <"
		else
			text = "T: "..UnitName(tunit)
		end

		self:AddLine(text, r, g, b)
	end
end

GameTooltip:HookScript("OnTooltipSetUnit", OnTooltipSetUnit)

--[[ Item Icons ]]
local frame = CreateFrame("Frame", "ItemRefTooltipIconFrame", _G["ItemRefTooltip"])
frame:SetPoint("TOPRIGHT", _G["ItemRefTooltip"], "TOPLEFT", -3, 0)
frame:SetHeight(32)
frame:SetWidth(32)

local tex = frame:CreateTexture("ItemRefTooltipIcon", "TOOLTIP")
tex:SetAllPoints(frame)

local bg = CreateFrame ("Frame", nil, frame)
bg:SetBackdrop({ 
	bgFile = FreeUI.backdrop, 
})

bg:SetBackdropColor(0, 0, 0)
bg:SetPoint("TOPLEFT", frame, -1, 1)
bg:SetPoint("BOTTOMRIGHT", frame, 1, -1)
bg:SetFrameStrata("LOW")
			
local AddItemIcon = function()
	local frame = _G["ItemRefTooltipIconFrame"]
	frame:Hide()

	local _, link = _G["ItemRefTooltip"]:GetItem()
	local icon = link and GetItemIcon(link)
	if(not icon) then return end

	_G["ItemRefTooltipIcon"]:SetTexture(icon)
	_G["ItemRefTooltipIcon"]:SetTexCoord(.08, .92, .08, .92)
	frame:Show()
end

hooksecurefunc("SetItemRef", AddItemIcon)