--[[-------------------------------------------------------------------------
  Base code from oUF_Lily, Edited by Haleth:
    Please ask for permission before redistributing this work.
---------------------------------------------------------------------------]]

local parent, ns = ...
local oUF = ns.oUF

local class = select(2, UnitClass("player"))

local colors = setmetatable({
	power = setmetatable({
		["MANA"] = {.9, .9, .9},
		["RAGE"] = {.9, .1, .1},
		["FUEL"] = {0, 0.55, 0.5},
		["FOCUS"] = {.9, .5, .1},
		["ENERGY"] = {.9, .9, .1},
		["AMMOSLOT"] = {0.8, 0.6, 0},
		["RUNIC_POWER"] = {.1, .9, .9},
		["POWER_TYPE_STEAM"] = {0.55, 0.57, 0.61},
		["POWER_TYPE_PYRITE"] = {0.60, 0.09, 0.17},
	}, {__index = oUF.colors.power}),
}, {__index = oUF.colors})

--[[ Dropdown menu ]]
local menu = function(self)
	local unit = self.unit:sub(1, -2)
	local cunit = self.unit:gsub("^%l", string.upper)

	if(cunit == 'Vehicle') then
		cunit = 'Pet'
	end

	if(unit == "party" or unit == "partypet") then
		ToggleDropDownMenu(1, nil, _G["PartyMemberFrame"..self.id.."DropDown"], "cursor", 0, 0)
	elseif(_G[cunit.."FrameDropDown"]) then
		ToggleDropDownMenu(1, nil, _G[cunit.."FrameDropDown"], "cursor", 0, 0)
	end
end

--[[ Short values ]]

local siValue = function(val)
	if(val >= 1e6) then
		return format("%.2fm", val * 0.000001)
	elseif(val >= 1e4) then
		return format("%.1fk", val * 0.001) 
	else
		return val
	end
end

--[[ Tags ]]

oUF.Tags['free:health'] = function(unit)
	if(not UnitIsConnected(unit) or UnitIsDead(unit) or UnitIsGhost(unit)) then return end

	local min, max = UnitHealth(unit), UnitHealthMax(unit)
	if(unit=="target") then
		return format("|cffffffff%s|r %.0f", siValue(min), (min/max)*100)
	else
		return siValue(min)
	end
end
oUF.TagEvents['free:health'] = oUF.TagEvents.missinghp

oUF.Tags['free:maxhealth'] = function(unit)
	if(not UnitIsConnected(unit) or UnitIsDead(unit) or UnitIsGhost(unit)) then return end

	local max = UnitHealthMax(unit)
	return max
end
oUF.TagEvents['free:maxhealth'] = oUF.TagEvents.missinghp

if FreeUI.config.healmode == true then
	oUF.Tags['free:missinghealth'] = function(unit)
		local min, max, name = UnitHealth(unit), UnitHealthMax(unit), UnitName(unit)
		if name and name:len() > 4 then name = name:sub(1, 4) end

		if not UnitIsConnected(unit) then
			return "Off" 
		elseif UnitIsDead(unit) then
			return "Dead"
		elseif UnitIsGhost(unit) then 
			return "Ghost"
		elseif min ~= max then
			return siValue(max-min)
		else
			return name
		end
	end
	oUF.TagEvents['free:missinghealth'] = oUF.TagEvents.missinghp
end

oUF.Tags['free:power'] = function(unit)
	local min, max = UnitPower(unit), UnitPowerMax(unit)
	if(min == 0 or max == 0 or not UnitIsConnected(unit) or UnitIsDead(unit) or UnitIsGhost(unit)) then return end

	return siValue(min)
end
oUF.TagEvents['free:power'] = oUF.TagEvents.missingpp

--[[ Update health bar colour ]]

local UpdateHealth = function(self, event, unit)
	if(self.unit == unit) then
		local r, g, b, t
		if(UnitIsTapped(unit) and not UnitIsTappedByPlayer(unit) or not UnitIsConnected(unit)) then
			r, g, b = .6, .6, .6
		elseif(unit == 'pet') then
			t = self.colors.happiness[GetPetHappiness()] or reactioncolors[UnitReaction(unit, "player")]
		elseif(UnitIsPlayer(unit)) then
			local _, class = UnitClass(unit)
			t = self.colors.class[class]
		else
			t = reactioncolors[UnitReaction(unit, "player")]
		end

		if(t) then
			r, g, b = t[1], t[2], t[3]
		end

		if(r) then
			if FreeUI.config.healmode == true and FreeUI.config.healmode_oldstyle == false then
				self.Health:SetStatusBarColor(0, 0, 0, 0)
				self.Power:SetStatusBarColor(r, g, b)
			else
				self.Health:SetStatusBarColor(r, g, b)
			end
		end

		if FreeUI.config.healmode == true and FreeUI.config.healmode_oldstyle == false then
			local min, max = UnitHealth(unit), UnitHealthMax(unit)
			self.Healthdef:SetPoint("LEFT", self.Health, self.Health:GetWidth() * (min/max), 0)
			if(not UnitIsConnected(unit) or UnitIsDead(unit) or UnitIsGhost(unit)) then
				self.Healthdef:Hide()
				if self.Text then self.Text:SetTextColor(.3, .7, 1) end
			else
				self.Healthdef:Show()
				self.Healthdef:SetVertexColor(self.ColorGradient(min / max, unpack(self.colors.smooth)))
				if self.Text then self.Text:SetTextColor(1, 1, 1) end
			end
		end
	end
end

--[[ Update health ]]

local PostUpdateHealth = function(Health, unit, min, max)
	if(UnitIsDead(unit)) then
		Health:SetValue(0)
	elseif(UnitIsGhost(unit)) then
		Health:SetValue(0)
	end

	if(Health.value) then
		if(unit=="target") then
			Health.value:SetTextColor(unpack(reactioncolors[UnitReaction("player", unit) or 5]))
		end
	end

	return UpdateHealth(Health:GetParent(), 'PostUpdateHealth', unit)
end

--[[ Hide Blizz frames ]]

CompactRaidFrameManager:SetAlpha(0)

_G["BuffFrame"]:Hide()

--[[ Aura icon filter ]]

local PostUpdateIcon
do
	local playerUnits = {
		player = true,
		pet = true,
		vehicle = true,
	}

	PostUpdateIcon = function(icons, unit, icon, index, offset, filter, isDebuff)
		local texture = icon.icon
		local name, _, _, _, dtype, duration, expirationTime, unitCaster = UnitAura(unit, index, icon.filter)
		if(unit~="target") then icon:Show() end
		if(UnitIsFriend("player", unit) or playerUnits[icon.owner] or FreeUI.debuffFilter[name] or not icon.debuff or UnitIsPlayer(unit)) then
			icon:Show()
		else
			icon:Hide()
		end
	end
end

--[[ Update power value ]]

local PostUpdatePower = function(Power, unit, min, max)
	local Health = Power:GetParent().Health
	if(min == 0 or max == 0 or not UnitIsConnected(unit)) then
		Power:SetValue(0)
	elseif(UnitIsDead(unit) or UnitIsGhost(unit)) then
		Power:SetValue(0)
	end
end

local RAID_TARGET_UPDATE = function(self, event)
	local index = GetRaidTargetIndex(self.unit)
	if(index) then
		self.RIcon:SetText(ICON_LIST[index].."22|t")
	else
		self.RIcon:SetText()
	end
end

--[[ Temp buffs ]]

for i = 1, 3 do
	_G["TempEnchant"..i.."Border"]:Hide()
	bu = _G["TempEnchant"..i]
	ic = _G["TempEnchant"..i.."Icon"]
	du = _G["TempEnchant"..i.."Duration"]

	bu:ClearAllPoints()
	bu:SetHeight(36)
	bu:SetWidth(36)

	if i==1 then
		bu:SetPoint("BOTTOMRIGHT", UIParent, "BOTTOMRIGHT", -6, 50)
	elseif i==2 then
		bu:SetPoint("BOTTOMRIGHT", UIParent, "BOTTOMRIGHT", -6, 88)
	else
		bu:SetPoint("BOTTOMRIGHT", UIParent, "BOTTOMRIGHT", -6, 126)
	end

	bu:SetBackdrop({
		edgeFile = FreeUI.backdrop, 
		edgeSize = 1, 
	})
	bu:SetBackdropBorderColor(0, 0, 0)

	ic:SetTexCoord(.08, .92, .08, .92)
	du:ClearAllPoints()
	du:SetPoint("BOTTOM", bu, 0, -2)
	du:SetFont(FreeUI.font, 8, "OUTLINEMONOCHROME")
	du:SetDrawLayer("OVERLAY")
	du:SetTextColor(1, 1, 1)
	du.SetTextColor = FreeUI.dummy
	du:SetShadowColor(0, 0, 0, 0)
end

--[[ Global ]]

local Shared = function(self, unit, isSingle)
	self.menu = menu

	self:SetScript("OnEnter", UnitFrame_OnEnter)
	self:SetScript("OnLeave", UnitFrame_OnLeave)

	-- XXX: Change to AnyUp when RegisterAttributeDriver doesn't cause clicks
	-- to get incorrectly eaten.
	self:RegisterForClicks"AnyDown"

	self.bg = CreateFrame("Frame", nil, self)
	self.bg:SetPoint("TOPLEFT", self, -1, 1)
	self.bg:SetPoint("BOTTOMRIGHT", self, 1, -1)
	self.bg:SetFrameStrata("BACKGROUND")
	FreeUI.CreateBD(self.bg)

	--[[ Health ]]

	local Health = CreateFrame("StatusBar", nil, self)
	Health:SetStatusBarTexture(FreeUI.texture)
	Health:SetFrameStrata("LOW")

	Health.frequentUpdates = true

	Health:SetPoint("TOP")
	Health:SetPoint("LEFT")
	Health:SetPoint("RIGHT")
	Health:SetPoint("BOTTOM", 0, 2)

	self.Health = Health

	--[[ Health deficit colour (thanks nin) ]]

	if FreeUI.config.healmode == true and FreeUI.config.healmode_oldstyle == false then
		local Healthdef = Health:CreateTexture(nil, "BORDER")
		Healthdef:SetTexture(FreeUI.texture)
		Healthdef:SetPoint("TOPRIGHT", Health)
		Healthdef:SetPoint("BOTTOMRIGHT", Health)
		Healthdef:SetPoint("LEFT", Health, 0, 0)

		self.Healthdef = Healthdef
	end

	--[[ Power ]]

	local Power = CreateFrame("StatusBar", nil, self)
	Power:SetStatusBarTexture(FreeUI.texture)
	Power:SetFrameStrata("LOW")

	if FreeUI.config.healmode == true and FreeUI.config.healmode_oldstyle == false then Power.colorPower = false else Power.colorPower = true end
	Power.frequentUpdates = true

	Power:SetHeight(1)

	Power:SetPoint("LEFT")
	Power:SetPoint("RIGHT")
	Power:SetPoint("TOP", Health, "BOTTOM", 0, -1)

	self.Power = Power

	local Powerbg = CreateFrame("Frame", nil, self)
	Powerbg:SetPoint("TOPLEFT", self.Power, -1, 1)
	Powerbg:SetPoint("BOTTOMRIGHT", self.Power, 1, -1)
	Powerbg:SetFrameStrata("BACKGROUND")
	FreeUI.CreateBD(Powerbg, 0)
	local _, powertype = UnitPowerType("player")
	if powertype == "RAGE" then
		Powerbg:SetBackdropColor(0, 0, 0, .3)
	end

	--[[ Castbar ]]

	local Castbar = CreateFrame("StatusBar", nil, self)
	Castbar:SetStatusBarTexture("")
	Castbar:SetToplevel(true)

	local Spark = Castbar:CreateTexture(nil, "OVERLAY")
	Spark:SetTexture"Interface\\CastingBar\\UI-CastingBar-Spark"
	Spark:SetBlendMode"ADD"
	Spark:SetWidth(16)
	Castbar.Spark = Spark

	self.Castbar = Castbar

	local PostCastStart = function(Castbar, unit, spell, spellrank)
		if Castbar.Text then Castbar.Text:SetText(spell) end

		if self.Iconbg then
			if Castbar.interrupt then
				self.Iconbg:SetBackdropBorderColor(1, 0, 0)
			else
				self.Iconbg:SetBackdropBorderColor(0, 0, 0)
			end
		end
	end

	local PostCastStop = function(Castbar, unit)
		if Castbar.Text then Castbar.Text:SetText("") end
	end

	local PostCastStopUpdate = function(self, event, unit)
		if(unit ~= self.unit) then return end
		return PostCastStop(self.Castbar, unit)
	end

	-- [[ Heal prediction ]]

	if FreeUI.config.healmode == true then
		local mhpb = CreateFrame("StatusBar", nil, self.Health)
		mhpb:SetPoint("TOPLEFT", self.Health:GetStatusBarTexture(), "TOPRIGHT", 0, 0)
		mhpb:SetPoint("BOTTOMLEFT", self.Health:GetStatusBarTexture(), "BOTTOMRIGHT", 0, 0)
		mhpb:SetWidth(229)
		mhpb:SetStatusBarTexture(FreeUI.texture)
		mhpb:SetStatusBarColor(0, .5, 1, 0.75)

		local ohpb = CreateFrame("StatusBar", nil, self.Health)
		ohpb:SetPoint("TOPLEFT", mhpb:GetStatusBarTexture(), "TOPRIGHT", 0, 0)
		ohpb:SetPoint("BOTTOMLEFT", mhpb:GetStatusBarTexture(), "BOTTOMRIGHT", 0, 0)
		ohpb:SetWidth(229)
		ohpb:SetStatusBarTexture(FreeUI.texture)
		ohpb:SetStatusBarColor(.5, 0, 1, 0.75)

		self.mhpb = mhpb
		self.ohpb = ohpb

		self.HealPrediction = {
			-- status bar to show my incoming heals
			myBar = mhpb,

			-- status bar to show other peoples incoming heals
			otherBar = ohpb,

			-- amount of overflow past the end of the health bar
			maxOverflow = 1,
		}
	end

	--[[ Set up the layout ]]

	self.colors = colors

	self.disallowVehicleSwap = true

	if(isSingle) then
		if(unit=="player" or unit=="target") then
			self:SetSize(229, 14)
		elseif(unit=="pet" or unit=="focus") then
			self:SetSize(113, 8)
		end
	end

	self:RegisterEvent("UNIT_NAME_UPDATE", PostCastStopUpdate)
	table.insert(self.__elements, PostCastStopUpdate)

	Castbar.PostChannelStart = PostCastStart
	Castbar.PostCastStart = PostCastStart

	Castbar.PostCastStop = PostCastStop
	Castbar.PostChannelStop = PostCastStop

	Health.PostUpdate = PostUpdateHealth
	Power.PostUpdate = PostUpdatePower
end

-- [[ Unit specific functions ]]

local UnitSpecific = {
	pet = function(self, ...)
		Shared(self, ...)

		local Health = self.Health
		local Power = self.Power
		local Castbar = self.Castbar
		local Spark = Castbar.Spark

		Health:SetHeight(6)

		Castbar:SetAllPoints(Health)
		Castbar.Width = self:GetWidth()

		Spark:SetHeight(self.Health:GetHeight())

		self:RegisterEvent("UNIT_HAPPINESS", UpdateHealth)
	end,

	player = function(self, ...)
		Shared(self, ...)

		local Health = self.Health
		local Power = self.Power
		local Castbar = self.Castbar
		local Spark = Castbar.Spark

		Health:SetHeight(12)

		local HealthPoints = FreeUI.CreateFS(Health, 8)
		local MaxHealthPoints = FreeUI.CreateFS(Health, 8, "RIGHT")

		HealthPoints:SetPoint("LEFT", self, "TOPLEFT", 0, 6)
		HealthPoints:SetJustifyH("LEFT")
		MaxHealthPoints:SetPoint("RIGHT", self, "TOPRIGHT", 0, 6)

		self:Tag(HealthPoints, '[dead][offline][free:health]')
		self:Tag(MaxHealthPoints, '[free:maxhealth]')
		Health.value = HealthPoints

		local PowerPoints = FreeUI.CreateFS(Power, 8)
		local _, UnitPowerType = UnitPowerType("player")
		if(UnitPowerType=="MANA") then
			PowerPoints:SetPoint("LEFT", self, "TOPLEFT", 34, 6)
			PowerPoints:SetTextColor(.4, .7, 1)
		end

		self:Tag(PowerPoints, '[free:power]')

		Power.value = PowerPoints

		if class == "DEATHKNIGHT" then
			local runes = CreateFrame("Frame", nil, self)
			runes:SetWidth(229)
			runes:SetHeight(2)
			runes:SetPoint("TOPLEFT", self, "BOTTOMLEFT", 0, -3)

			local rbd = CreateFrame("Frame", nil, runes)
			rbd:SetBackdrop({
				edgeFile = FreeUI.backdrop,
				edgeSize = 1,
			})
			rbd:SetBackdropBorderColor(0, 0, 0)
			rbd:SetPoint("TOPLEFT", runes, -1, 1)
			rbd:SetPoint("BOTTOMRIGHT", runes, 1, -1)

			for i = 1, 6 do
				runes[i] = CreateFrame("StatusBar", nil, self)
				runes[i]:SetHeight(2)
				runes[i]:SetStatusBarTexture(FreeUI.texture)
				runes[i]:SetStatusBarColor(255/255,101/255,101/255)

				local rbd = CreateFrame("Frame", nil, runes[i])
				rbd:SetBackdrop({
					edgeFile = FreeUI.backdrop,
					edgeSize = 1,
				})
				rbd:SetBackdropBorderColor(0, 0, 0)
				rbd:SetPoint("TOPLEFT", runes[i], -1, 1)
				rbd:SetPoint("BOTTOMRIGHT", runes[i], 1, -1)

				if i == 1 then
					runes[i]:SetPoint("LEFT", runes)
					runes[i]:SetWidth(38)
				elseif i == 2 then
					runes[i]:SetPoint("LEFT", runes[i-1], "RIGHT", 1, 0)
					runes[i]:SetWidth(38)
				else
					runes[i]:SetPoint("LEFT", runes[i-1], "RIGHT", 1, 0)
					runes[i]:SetWidth(37)
				end
			end

			self.Runes = runes
		elseif class == "DRUID" then
			local eclipseBar = CreateFrame("Frame", nil, self)
			eclipseBar:SetWidth(229)
			eclipseBar:SetHeight(2)
			eclipseBar:SetPoint("TOPLEFT", self, "BOTTOMLEFT", 0, -3)

			local ebd = CreateFrame("Frame", nil, eclipseBar)
			ebd:SetBackdrop({
				edgeFile = FreeUI.backdrop,
				edgeSize = 1,
			})
			ebd:SetBackdropBorderColor(0, 0, 0)
			ebd:SetPoint("TOPLEFT", eclipseBar, -1, 1)
			ebd:SetPoint("BOTTOMRIGHT", eclipseBar, 1, -1)
			ebd:SetFrameStrata("BACKGROUND")

			local glow = CreateFrame("Frame", nil, eclipseBar)
			glow:SetBackdrop({
				edgeFile = FreeUI.glow,
				edgeSize = 5,
			})
			glow:SetPoint("TOPLEFT", eclipseBar, -6, 6)
			glow:SetPoint("BOTTOMRIGHT", eclipseBar, 6, -6)

			local hasEclipse = function(self, unit)
				if self.hasSolarEclipse then
					glow:SetBackdropBorderColor(.30, .52, .90, 1)
				elseif self.hasLunarEclipse then
					glow:SetBackdropBorderColor(.80, .82, .60, 1)
				else
					glow:SetBackdropBorderColor(0, 0, 0, 0)
				end
			end 

			local LunarBar = CreateFrame("StatusBar", nil, eclipseBar)
			LunarBar:SetPoint("LEFT", eclipseBar, "LEFT", 0, 0)
			LunarBar:SetSize(eclipseBar:GetWidth(), eclipseBar:GetHeight())
			LunarBar:SetStatusBarTexture(FreeUI.texture)
			LunarBar:SetStatusBarColor(.30, .52, .90)
			LunarBar:SetFrameStrata("LOW")
			eclipseBar.LunarBar = LunarBar

			local SolarBar = CreateFrame("StatusBar", nil, eclipseBar)
			SolarBar:SetPoint("LEFT", LunarBar:GetStatusBarTexture(), "RIGHT", 0, 0)
			SolarBar:SetSize(eclipseBar:GetWidth(), eclipseBar:GetHeight())
			SolarBar:SetStatusBarTexture(FreeUI.texture)
			SolarBar:SetStatusBarColor(.80, .82, .60)
			SolarBar:SetFrameStrata("LOW")
			eclipseBar.SolarBar = SolarBar

			local eclipseBarText = FreeUI.CreateFS(eclipseBar, 24)
			eclipseBarText:SetPoint("LEFT", self, "RIGHT", 10, 0)
			self:Tag(eclipseBarText, '[pereclipse]')

			self.EclipseBar = eclipseBar

			self.EclipseBar.PostUnitAura = hasEclipse

			self.EclipseBar.PostUpdatePower = function(self)
				if GetEclipseDirection() == "sun" then
					eclipseBarText:SetTextColor(.80, .82, .60)
				elseif GetEclipseDirection() == "moon" then
					eclipseBarText:SetTextColor(.30, .52, .90)
				else
					eclipseBarText:SetTextColor(1, 1, 1)
				end
			end
		elseif class == "PALADIN" then
			local UpdateHoly = function(self, event, unit, powerType)
				if(self.unit ~= unit or (powerType and powerType ~= 'HOLY_POWER')) then return end
				local num = UnitPower(unit, SPELL_POWER_HOLY_POWER)
				for i = 1, MAX_HOLY_POWER do
					if(i <= num) then
						self.glow:SetAlpha(1)
						FreeUI.CreatePulse(self.glow)
						self.count:SetText(num)
						self.count:SetTextColor(1, 1, 0)
						self.count:SetFont(FreeUI.font, 40, "OUTLINEMONOCHROME")
					elseif num == 0 then
						self.glow:SetScript("OnUpdate", nil)
						self.glow:SetAlpha(0)
						self.count:SetText("")
					else
						self.glow:SetScript("OnUpdate", nil)
						self.glow:SetAlpha(0)
						self.count:SetText(num)
						self.count:SetTextColor(1, 1, 1)
						self.count:SetFont(FreeUI.font, 24, "OUTLINEMONOCHROME")
					end
				end
			end

			local glow = CreateFrame("Frame", nil, self)
			glow:SetBackdrop({
				edgeFile = FreeUI.glow,
				edgeSize = 5,
			})
			glow:SetPoint("TOPLEFT", self, -6, 6)
			glow:SetPoint("BOTTOMRIGHT", self, 6, -6)
			glow:SetBackdropBorderColor(228/255,225/255,16/255)

			self.glow = glow

			local count = FreeUI.CreateFS(self, 24 * UIParent:GetScale())
			count:SetPoint("LEFT", self, "RIGHT", 10, 0)

			self.count = count

			self.HolyPower = glow
			glow.Override = UpdateHoly
		elseif class == "WARLOCK" then
			local bars = CreateFrame("Frame", nil, self)
			bars:SetWidth(229)
			bars:SetHeight(1)
			bars:SetPoint("TOPLEFT", self, "BOTTOMLEFT", 0, -3)

			local bbd = CreateFrame("Frame", nil, bars)
			bbd:SetBackdrop({
				bgFile = FreeUI.backdrop,
				edgeFile = FreeUI.backdrop,
				edgeSize = 1,
			})
			bbd:SetBackdropColor(0, 0, 0, .5)
			bbd:SetBackdropBorderColor(0, 0, 0)
			bbd:SetPoint("TOPLEFT", bars, -1, 1)
			bbd:SetPoint("BOTTOMRIGHT", bars, 1, -1)
			bbd:SetFrameStrata("BACKGROUND")

			for i = 1, 3 do
				bars[i] = CreateFrame("StatusBar", nil, self)
				bars[i]:SetHeight(1)
				bars[i]:SetStatusBarTexture(FreeUI.texture)
				bars[i]:SetStatusBarColor(255/255,101/255,101/255)

				local bbd = CreateFrame("Frame", nil, bars[i])
				bbd:SetBackdrop({
					edgeFile = FreeUI.backdrop,
					edgeSize = 1,
				})
				bbd:SetBackdropBorderColor(0, 0, 0)
				bbd:SetPoint("TOPLEFT", bars[i], -1, 1)
				bbd:SetPoint("BOTTOMRIGHT", bars[i], 1, -1)

				if i == 1 then
					bars[i]:SetPoint("LEFT", bars)
					bars[i]:SetWidth(75)
				else
					bars[i]:SetPoint("LEFT", bars[i-1], "RIGHT", 1, 0)
					bars[i]:SetWidth(76)
				end
			end

			self.SoulShards = bars
		end

		Castbar.Width = self:GetWidth()
		Spark:SetHeight(self.Health:GetHeight())
		Castbar.Text = FreeUI.CreateFS(Castbar, 8)

		local IconFrame = CreateFrame("Frame", nil, Castbar)

		local Icon = IconFrame:CreateTexture(nil, "BACKGROUND")
		Icon:SetAllPoints(IconFrame)
		Icon:SetTexCoord(.08, .92, .08, .92)

		Castbar.Icon = Icon

		self.Iconbg = CreateFrame("Frame", nil, IconFrame)
			self.Iconbg:SetBackdrop({
			edgeFile = FreeUI.backdrop, 
			edgeSize = 1, 
		})
		self.Iconbg:SetPoint("TOPLEFT", IconFrame, "TOPLEFT", -1 , 1)
		self.Iconbg:SetPoint("BOTTOMRIGHT", IconFrame, "BOTTOMRIGHT", 1, -1)
		self.Iconbg:SetFrameStrata("LOW")

		if FreeUI.config.castbar == 2 then
			Castbar:SetStatusBarTexture(FreeUI.texture)
			Castbar:SetStatusBarColor(0.26, 1, 0.22)
			Castbar:SetWidth(self:GetWidth())
			Castbar:SetHeight(self:GetHeight())
			Castbar:SetPoint("BOTTOM", UIParent, "BOTTOM", 0, 420)
			Castbar.Text:SetAllPoints(Castbar)
			local sf = Castbar:CreateTexture(nil, "OVERLAY")
			Castbar.SafeZone = sf
			IconFrame:SetPoint("LEFT", Castbar, "RIGHT", 3, 0)
			IconFrame:SetSize(22, 22)

			local bg = CreateFrame("Frame", nil, Castbar)
			bg:SetBackdrop({
				bgFile = FreeUI.backdrop,
				edgeFile = FreeUI.backdrop, 
				edgeSize = 1, 
			})
			bg:SetPoint("TOPLEFT", Castbar, -1, 1)
			bg:SetPoint("BOTTOMRIGHT", Castbar, 1, -1)
			bg:SetBackdropColor(0, 0, 0, .5)
			bg:SetBackdropBorderColor(0, 0, 0)
			bg:SetFrameStrata("BACKGROUND")
		else
			Castbar:SetAllPoints(Health)
			Castbar.Text:SetAllPoints(Health)
			IconFrame:SetPoint("RIGHT", self, "LEFT", -10, 0)
			IconFrame:SetSize(44, 44)
		end

		local Buffs = CreateFrame("Frame", nil, self)
		Buffs.initialAnchor = "TOPRIGHT"

		Buffs:SetPoint("TOPRIGHT", UIParent, "TOPRIGHT", -50, -50)

		local RaidIcon = FreeUI.CreateFS(Health, 12)
		RaidIcon:SetPoint("RIGHT", self, "LEFT", -3, 0)

		self.RIcon = RaidIcon
		self:RegisterEvent("RAID_TARGET_UPDATE", RAID_TARGET_UPDATE)
		table.insert(self.__elements, RAID_TARGET_UPDATE)

		Buffs["growth-y"] = "DOWN"
		Buffs["growth-x"] = "LEFT"
		Buffs["spacing-x"] = 3
		Buffs["spacing-y"] = 3

		Buffs:SetHeight(500)
		Buffs:SetWidth(310)
		Buffs.size = 32

		self.Buffs = Buffs

		local Debuffs = CreateFrame("Frame", nil, self)
		Debuffs.initialAnchor = "TOPRIGHT"
		if class == "DEATHKNIGHT" or class == "DRUID" or class == "WARLOCK" then
			Debuffs:SetPoint("TOPRIGHT", self, "BOTTOMRIGHT", 0, -11)
		else
			Debuffs:SetPoint("TOPRIGHT", self, "BOTTOMRIGHT", 0, -6)
		end
		Debuffs["growth-x"] = "LEFT"
		Debuffs["growth-y"] = "DOWN"
		Debuffs['spacing-x'] = 3
		Debuffs['spacing-y'] = 3

		Debuffs:SetHeight(60)
		Debuffs:SetWidth(229)
		Debuffs.num = 8
		Debuffs.size = 26

		self.Debuffs = Debuffs

		Debuffs.PostUpdateIcon = PostUpdateIcon

		Buffs.PostUpdateIcon = PostUpdateIcon
	end,

	target = function(self, ...)
		Shared(self, ...)

		local Health = self.Health
		local Power = self.Power
		local Castbar = self.Castbar
		local Spark = Castbar.Spark

		Health:SetHeight(12)

		local HealthPoints = FreeUI.CreateFS(Health, 8)

		HealthPoints:SetPoint("LEFT", self, "TOPLEFT", 0, 6)
		HealthPoints:SetJustifyH("LEFT")

		self:Tag(HealthPoints, '[dead][offline][free:health]')
		Health.value = HealthPoints

		local PowerPoints = FreeUI.CreateFS(Power, 8)
		PowerPoints:SetPoint("BOTTOMLEFT", Health.value, "BOTTOMRIGHT")

		self:Tag(PowerPoints, '[free:power]')

		Power.value = PowerPoints

		Castbar:SetAllPoints(Health)
		Castbar.Width = self:GetWidth()

		Spark:SetHeight(self.Health:GetHeight())

		Castbar.Text = FreeUI.CreateFS(Health, 8)
		Castbar.Text:SetAllPoints(Health)

		local IconFrame = CreateFrame("Frame", nil, Castbar)
		IconFrame:SetPoint("LEFT", self, "RIGHT", 3, 0)
		IconFrame:SetHeight(44)
		IconFrame:SetWidth(44)

		local Icon = IconFrame:CreateTexture(nil, "BACKGROUND")
		Icon:SetAllPoints(IconFrame)
		Icon:SetTexCoord(.08, .92, .08, .92)

		Castbar.Icon = Icon

		self.Iconbg = CreateFrame("Frame", nil, IconFrame)
		self.Iconbg:SetBackdrop({
			bgFile = "",
			edgeFile = FreeUI.backdrop, 
			edgeSize = 1, 
			insets = {left = 0, right = 0, top = 0, bottom = 0}
		})
		self.Iconbg:SetPoint("TOPLEFT", IconFrame, "TOPLEFT", -1 , 1)
		self.Iconbg:SetPoint("BOTTOMRIGHT", IconFrame, "BOTTOMRIGHT", 1, -1)
		self.Iconbg:SetFrameStrata("LOW")

		local Name = FreeUI.CreateFS(Health, 8)
		Name:SetPoint("BOTTOMLEFT", Power.value, "BOTTOMRIGHT")
		Name:SetPoint("RIGHT", self)
		Name:SetJustifyH"RIGHT"
		Name:SetTextColor(1, 1, 1)

		self:Tag(Name, '[name]')
		self.Name = Name

		local RaidIcon = FreeUI.CreateFS(Health, 12)
		RaidIcon:SetPoint("RIGHT", self, "LEFT", -3, 0)

		self.RIcon = RaidIcon
		self:RegisterEvent("RAID_TARGET_UPDATE", RAID_TARGET_UPDATE)
		table.insert(self.__elements, RAID_TARGET_UPDATE)

		local Auras = CreateFrame("Frame", nil, self)
		Auras:SetPoint("TOPLEFT", self, "BOTTOMLEFT", 0, -6)
		Auras.initialAnchor = "TOPLEFT"
		Auras["growth-x"] = "RIGHT"
		Auras["growth-y"] = "DOWN"
		Auras['spacing-x'] = 3
		Auras['spacing-y'] = 3
		Auras.numDebuffs = 16
		Auras.numBuffs = 16
		Auras:SetHeight(500)
		Auras:SetWidth(229)
		Auras.size = 26
		Auras.gap = true

		self.Auras = Auras

		Auras.PostUpdateIcon = PostUpdateIcon
	end,

	focus = function(self, ...)
		Shared(self, ...)

		local Health = self.Health
		local Power = self.Power
		local Castbar = self.Castbar
		local Spark = Castbar.Spark

		Health:SetHeight(6)

		Castbar:SetAllPoints(Health)
		Castbar.Width = self:GetWidth()

		Spark:SetHeight(Health:GetHeight())

		local Debuffs = CreateFrame("Frame", nil, self)
		Debuffs:SetPoint("CENTER", UIParent, "CENTER", -216, -56)
		Debuffs.initialAnchor = "BOTTOMLEFT"
		Debuffs["growth-x"] = "RIGHT"
		Debuffs["growth-y"] = "DOWN"
		Debuffs["spacing-x"] = 3
		Debuffs:SetHeight(22)
		Debuffs:SetWidth(115)
		Debuffs.size = 22
		Debuffs.num = 5
		self.Debuffs = Debuffs

		Debuffs.PostUpdateIcon = PostUpdateIcon
	end,
}

do
	UnitSpecific.boss = function(self)
		if unit and unit:find("boss%d") then
			Shared(self)

			self:SetAttribute("type2", "focus")

			local Health, Power = self.Health, self.Power

			self:SetAttribute('initial-height', 12)
			self:SetAttribute('initial-width', 126)

			Health:SetHeight(10)

			local RaidIcon = FreeUI.CreateFS(Health, 8)
			RaidIcon:SetPoint("CENTER")

			self.RIcon = RaidIcon
			self:RegisterEvent("RAID_TARGET_UPDATE", RAID_TARGET_UPDATE)
			table.insert(self.__elements, RAID_TARGET_UPDATE)
		end
	end
end

do
	local range = {
		insideAlpha = 1,
		outsideAlpha = .5,
	}

	UnitSpecific.party = function(self, ...)
		Shared(self, ...)

		self.disallowVehicleSwap = false

		local Health, Power = self.Health, self.Power
		if FreeUI.config.healmode == true then
			self:SetSize(42, 42)
			Health:SetHeight(40)
			self.mhpb:SetWidth(40)
			self.ohpb:SetWidth(40)
		else
			self:SetSize(38, 22)
			Health:SetHeight(20)
		end

		if FreeUI.config.healmode == true then
			local Text = FreeUI.CreateFS(Health, 8, "CENTER")
			if FreeUI.config.healmode_oldstyle == true then Text:SetPoint("CENTER", 0, 0) else Text:SetPoint("CENTER", 1, 0) end
			self.Text = Text

			self:Tag(Text, '[free:missinghealth]')
		else
			local NotHere = FreeUI.CreateFS(Health, 8, "CENTER")
			NotHere:SetPoint("CENTER", 1, 0)

			self:Tag(NotHere, '[dead][offline]')
		end

		local RaidIcon = FreeUI.CreateFS(Health, 8)
		RaidIcon:SetPoint("CENTER")

		self.RIcon = RaidIcon
		self:RegisterEvent("RAID_TARGET_UPDATE", RAID_TARGET_UPDATE)
		table.insert(self.__elements, RAID_TARGET_UPDATE)

		local Leader = FreeUI.CreateFS(self, 8, "LEFT")
		Leader:SetText("l")
		Leader:SetPoint("TOPLEFT", Health, 1, -1)

		self.Leader = Leader

		local MasterLooter = FreeUI.CreateFS(self, 8, "RIGHT")
		MasterLooter:SetText("m")
		MasterLooter:SetPoint("TOPRIGHT", Health, 1, 0)

		self.MasterLooter = MasterLooter

		local rc = Health:CreateTexture(nil, "OVERLAY")
		rc:SetPoint"TOPLEFT"
		rc:SetHeight(16)
		rc:SetWidth(16)
		self.ReadyCheck = rc

		local UpdateLFD = function(self, event)
			local lfdrole = self.LFDRole

			local role = UnitGroupRolesAssigned(self.unit)

			if role == "DAMAGER" then
				lfdrole:SetTextColor(1, .1, .1, 1)
			elseif role == "TANK" then
				lfdrole:SetTextColor(.3, .4, 1, 1)
			elseif role == "HEALER" then
				lfdrole:SetTextColor(0, 1, 0, 1)
			else
				lfdrole:SetTextColor(0, 0, 0, 0)
			end
		end

		local lfd = FreeUI.CreateFS(Health, 16, "CENTER")
		lfd:SetPoint("BOTTOM", 1, -4)
		lfd:SetText("•")
		lfd.Override = UpdateLFD

		self.LFDRole = lfd

		local dbh = Health:CreateTexture(nil, "OVERLAY")
		dbh:SetTexture(FreeUI.questborder)
		dbh:SetVertexColor(0, 0, 0, 1)
		dbh.DebuffHighlightAlpha = 1
		dbh:SetBlendMode("ADD")
		dbh:SetPoint("TOPLEFT", self, -3, 3)
		dbh:SetPoint("BOTTOMRIGHT", self, 3, -3)
		self.DebuffHighlight = dbh
		self.DebuffHighlightFilter = true

		if(class == "DRUID" or class == "PALADIN" or class == "PRIEST" or class == "SHAMAN") and FreeUI.config.healmode == true then
			local auras = {}
			local spellIDs = { 
				--Druid spells 
				774, --Rejuvenation 
				8936, --Regrowth 
				33763, --Lifebloom 
				48438, --Wild Growth

				--Priest spells 
				6788,  --Weakened Soul 
				33076, --Prayer of Mending 
				139, --Renew 
				17, --Power Word: Shield

				--Shaman spells 
				61295, --Riptide  
				16236, --Ancestral Fortitude

				--Paladin spells 
				53563, --Beacon of Light 
			}

			auras.onlyShowPresent = true
			auras.icons = {}
			for i, sid in pairs(spellIDs) do
				local icon = CreateFrame("Frame", nil, self)
				icon.spellID = sid
				icon:SetWidth(12)
				icon:SetHeight(12)

				if i == 1 or i == 5 or i == 9 or i == 11 then
					icon:SetPoint("BOTTOMLEFT", Health, "BOTTOMLEFT", 1, 1)
				elseif i == 2 or i == 6 or i == 10 then
					icon:SetPoint("BOTTOMRIGHT", Health, "BOTTOMRIGHT", -1, 1)
				elseif i == 3 or i == 7 then
					icon:SetPoint("TOPLEFT", Health, "TOPLEFT", 1, -1)
				elseif i == 4 or i == 8 then
					icon:SetPoint("TOPRIGHT", Health, "TOPRIGHT", -1, -1)
				end

				auras.icons[sid] = icon

				local cd = CreateFrame("Cooldown", nil, icon)
				cd:SetAllPoints(icon)
				cd:SetReverse()
				icon.cd = cd

				local count = FreeUI.CreateFS(icon, 8)
				count:SetPoint("TOP", icon, "TOP", 0, -2)
				icon.count = count

				FreeUI.CreateBD(icon)
			end
			self.AuraWatch = auras
		end

		local Threat = FreeUI.CreateFS(Health, 8)
		Threat:SetText("x")
		Threat:SetPoint("TOP", self, "TOP", 0, 0)
		self.Threat = Threat

		self.Range = range
	end
end

--[[ Register and activate style ]]

oUF:RegisterStyle("Free", Shared)
for unit,layout in next, UnitSpecific do
	oUF:RegisterStyle('Free - ' .. unit:gsub("^%l", string.upper), layout)
end

local spawnHelper = function(self, unit, ...)
	if(UnitSpecific[unit]) then
		self:SetActiveStyle('Free - ' .. unit:gsub("^%l", string.upper))
		local object = self:Spawn(unit)
		object:SetPoint(...)
		return object
	else
		self:SetActiveStyle'Free'
		local object = self:Spawn(unit)
		object:SetPoint(...)
		return object
	end
end

oUF:Factory(function(self)
	spawnHelper(self, 'focus', "BOTTOMLEFT", UIParent, "BOTTOMLEFT", 566, 447)
	spawnHelper(self, 'pet', "BOTTOMLEFT", UIParent, "BOTTOMLEFT", 450, 447)
	spawnHelper(self, 'player', "BOTTOMLEFT", UIParent, "BOTTOMLEFT", 450, 420)
	spawnHelper(self, 'target', "TOP", UIParent, "BOTTOM", 0, 300)

	self:SetActiveStyle'Free - Party'

	local party = self:SpawnHeader(
		nil, nil, 'party,solo', 
		'showParty', true, 
		'showPlayer', true and FreeUI.config.healmode == true or false, 
		'showSolo', false, 
		'yoffset', -3, 
		'maxColumns', 5, 
		'unitsperColumn', 1, 
		'columnSpacing', 3, 
		'columnAnchorPoint', "RIGHT"
	)

	if FreeUI.config.healmode == false then
		party:SetPoint("BOTTOM", oUF_FreePlayer, "TOP", 0, 50)
	else
		party:SetPoint("CENTER", 280, -70)
	end

	local raid = self:SpawnHeader(nil, nil, 'raid,party,solo',
		'showPlayer', true,
		'showSolo', false,
		'showParty', false,
		'showRaid', true,
		'xoffset', 5,
		'yOffset', -4,
		'point', "TOP",
		'groupFilter', '1,2,3,4,5,6,7,8',
		'groupingOrder', '1,2,3,4,5,6,7,8',
		'groupBy', 'GROUP',
		'maxColumns', 7,
		'unitsPerColumn', 5,
		'columnSpacing', 5,
		'columnAnchorPoint', "RIGHT"
	)

	if FreeUI.config.healmode == false then
		raid:SetPoint("BOTTOMRIGHT", Minimap, "BOTTOMLEFT", -5, 0)
	else
		raid:SetPoint("CENTER", 280, -70)
	end

	for i = 1,MAX_BOSS_FRAMES do
		local t_boss = _G["Boss"..i.."TargetFrame"]
		t_boss:UnregisterAllEvents()
		t_boss.Show = FreeUI.dummy
		t_boss:Hide()
		_G["Boss"..i.."TargetFrame".."HealthBar"]:UnregisterAllEvents()
		_G["Boss"..i.."TargetFrame".."ManaBar"]:UnregisterAllEvents()
	end

	local boss = {}
	for i = 1, MAX_BOSS_FRAMES do
		boss[i] = self:Spawn("boss"..i, "oUF_FreeBoss"..i)
		if i == 1 then
			boss[i]:SetPoint("LEFT", UIParent, "LEFT", 75, 0)
		else
			boss[i]:SetPoint("TOPLEFT", boss[i-1], "BOTTOMLEFT", 0, -40)
		end
	end
end)

--[[ Targettarget text ]]

local tt = CreateFrame("Frame", nil, UIParent)
tt:SetPoint("BOTTOMLEFT", UIParent, "BOTTOM", 5, 314)
tt:SetWidth(110)
tt:SetHeight(12)

ttt = FreeUI.CreateFS(tt, 8, "RIGHT")
ttt:SetAllPoints(tt)

tt:SetScript("OnUpdate", function()
	if(UnitName("targettarget")==UnitName("player")) then
		ttt:SetText("> YOU <")
		ttt:SetTextColor(1, 0, 0)
	else
		ttt:SetText(UnitName"targettarget")
		ttt:SetTextColor(1, 1, 1)
	end
end)